#define DISPLAYNAME_PLUGIN_ID	"Display Name"
#include <stdio.h>
#include <string.h>

#include "internal.h"
#include "connection.h"
#include "core.h"
#include "debug.h"
#include "signals.h"
#include "notify.h"

#define CHANGE_INTERVAL (10*1000)

static guint timer_id;

static gboolean
displayname_changer(void *data)
{
	FILE *data_fp;
	char name[256]={(char)NULL,};
	static char prev_name[255]={(char)NULL,};
	GaimConnection *gc = (GaimConnection *)data;
	GaimPluginProtocolInfo *prpl_info;
	prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
	gaim_debug(GAIM_DEBUG_INFO, "displayname", "Changer called\n");

	data_fp = fopen("/tmp/gaim_name","r");
	if(data_fp != NULL) {
		gaim_debug(GAIM_DEBUG_INFO, "displayname", 
					"Succeed to open a file\n");
		fread(name, 255, 1, data_fp);
		if(ferror(data_fp)==0) {
			gaim_debug(GAIM_DEBUG_INFO, "displayname",
				   "Finish to read the name from a file\n");
			/* Save useless "REA" reqeust */
			if(memcmp(prev_name,name,255) != 0) {
				gaim_debug(GAIM_DEBUG_INFO, "displayname",
						     "Change the name!!!\n");
				memcpy(prev_name, name, 255);
				gaim_connection_set_display_name(gc,name);
				prpl_info->set_info(gc,name);
			} 
			fclose(data_fp);
		} else {
			gaim_debug(GAIM_DEBUG_INFO, "displayname",
				"Failed to read a name from list\n");
		}
	} 
	return TRUE;
}

static void
signed_on_cb(GaimConnection *gc, void *data)
{
	GaimPluginProtocolInfo *prpl_info;
	prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
	if(prpl_info == NULL) {
		gaim_notify_error(gc, NULL, 
				"Couldn't get the plugin info.", NULL);
	} else {
		/* Set timer to change display name */
		timer_id = g_timeout_add(CHANGE_INTERVAL,
		       	displayname_changer, (gpointer *)gc);
	}
}

static void
signed_off_cb(GaimConnection *gc, void *data)
{
	if(timer_id > 0) {
		g_source_remove(timer_id);
	}
}

static void
quitting_cb(void *data)
{
	if(timer_id > 0) {
		g_source_remove(timer_id);
	}
}

/* 
 * FIXME: If U didn't load this plugin after signed on, this will not work.
 * 	  So, if you have any idea, tell me. or fix it yourself
 */
static gboolean plugin_load(GaimPlugin *plugin)
{
	void *conn_handle = gaim_connections_get_handle();
	void *core_handle = gaim_get_core();
	/* I think, we should to connect more signal 
	 * to making more completely */
	gaim_signal_connect(conn_handle, "signed-on",
				plugin, GAIM_CALLBACK(signed_on_cb), NULL);
	gaim_signal_connect(conn_handle, "signed-off",
				plugin, GAIM_CALLBACK(signed_off_cb), NULL);
	gaim_signal_connect(core_handle, "quitting",
				plugin, GAIM_CALLBACK(quitting_cb), NULL);
	return TRUE;
}

static void plugin_unload(void)
{
	if(timer_id > 0) {
		g_source_remove(timer_id);
	}
	return;
}

static GaimPluginInfo info =
{
	2,                     /**< api_version    */
	GAIM_PLUGIN_STANDARD,  /**< type           */
	NULL,                  /**< ui_requirement */
	0,                     /**< flags          */
	NULL,                  /**< dependencies   */
	GAIM_PRIORITY_DEFAULT, /**< priority       */

	DISPLAYNAME_PLUGIN_ID, /**< id             */
	"Displayname",         /**< name           */
	VERSION,    	       /**< version        */
	/**  summary                               */
	"Changing displyname automaticaly",
	/**  description                           */
	"This plugin work for changing displayname from outside of gaim" \
	   "(using pipe)",
	"Park, Sung-Jae <darkpgm@kornet.net>,"\
	   "Yang, Dae-Hyun <fasthyun@magicn.com>",
	"http://sjpark.coolfam.net",
	plugin_load,
	plugin_unload,
	NULL,		       /**< destroy        */
	NULL,      	       /**< ui_info        */
	NULL                   /**< extra_info     */
};

static void init_plugin(GaimPlugin *plugin) { }

GAIM_INIT_PLUGIN(displayname, init_plugin, info);
